"""
WebFrane language translator module.

This system is used by the template engine to decode and translate language
tagged texts in the following format:
    [%TAG Untranslated text%]

The "TAG" is looked up in the database and current language and if it
exists the entire [%...%] block will be replaced by the translation.
If it is not present the block will be replaced by the untranslated
text.

If you need to translate something in python code you should use a
construct like

   trans = Translate.translate("[%TAG Untranslated%]")

instead of using the get() function because this makes it a lot easier
to write scripts that process all .py and template files to populate
the database and create translation files. This way you don't have to
keep track of what tags you use yourself.

The translation system utilizes the cache (WebFrame.cache) and cache
daemon to speed up translation tag lookup. Because of this it is very
important that you do not change translations directly in the
database but instead use the set() function. Or that you make sure to
restart apache and kill the cache daemon after you have updated the db.
"""

import re
import WebFrame as WF
from WebFrame import cache

__pattern = re.compile("\[%(\w+)\s*(.*?)%]", re.UNICODE | re.MULTILINE)

def get_db(ctx, tag, lang):
    """Get a translation tag from the database, bypassing the cache
    """
    dbh = WF.open_db()
    cr = dbh.simple(WF.get_opt("lang.sql_get"), {"tag":tag, "lang":lang})
    if cr.rowcount == 0:
        return None
    else:
        out = cr.fetchone()
        return out["text"]

def get(ctx, tag, text=None, lang=None):
    """Get a translation tag from the cache (or database)
    If the tag is not found then the contents of 'text' is returned.
    If lang is not specified the language from the context is used"""
    if lang == None:
        lang = ctx.lang

    key = lang+"-"+tag
    if cache.has("__WF_TRANSLATION", key):
        out = cache.get("__WF_TRANSLATION", key)
    else:
        # Not found in cache, delegate to db and set in cache
        out = get_db(ctx, tag, lang)
        cache.set("__WF_TRANSLATION", key, out, local=True)

    # If out is none, send text
    if out == None:
        return text
    else:
        return out

def set_db(ctx, tag, text, lang):
    """Get a translation tag from the database, bypassing the cache
    """
    dbh = WF.open_db()
    cr = dbh.simple(WF.get_opt("lang.sql_set"), {"tag":tag,
                                                 "lang":lang,
                                                 "text":text})

def set(ctx, tag, text, lang=None):
    """Set/update a translation tag in the cache and db"""
    
    if lang == None:
        lang = ctx.lang

    key = lang+"-"+tag
    cache.set("__WF_TRANSLATION", key, text, local=True)
    set_db(ctx, tag, text, lang)

def translate(ctx, text, lang=None):
    """Translate a tagged string of the format "[%TAG String%]" to
    its translation in the current language"""
    global __pattern
    
    out = ""
    pos = 0
    for m in __pattern.finditer(text):
        out += text[pos:m.start()]
        pos = m.end()
        out += get(ctx, m.group(1), text=m.group(2), lang=lang)
        
    out += text[pos:]
    return out

